#ifndef SAVING_H
#define SAVING_H

//////////////////////////////////////////////////////////////////////
// TYPES ET DEFINITIONS
//////////////////////////////////////////////////////////////////////
#define SRAM 0xE000000

//////////////////////////////////////////////////////////////////////
// PROTOTYPES
//////////////////////////////////////////////////////////////////////
void saveByte(u16, u8);
void saveString(u16, char *);
void saveInt(u16, int);
u8   loadByte(u16);
int  loadInt(u16);
void loadString(u16, int, char *);

//////////////////////////////////////////////////////////////////////
// IMPLEMENTATIONS
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Sauvegarde un octet.
//
//////////////////////////////////////////////////////////////////////
void saveByte(u16 offset, u8 value)
{
  *(u8 *) (SRAM + offset) = value;

} // fin saveByte()

//////////////////////////////////////////////////////////////////////
// Sauvegarde une chane.
//
//////////////////////////////////////////////////////////////////////
void saveString(u16 offset, char *string)
{
  int i = 0;
  while (string[i] != '\0')
  {
    *(u8 *) (SRAM + offset + i) = string[i];
    i++;
  }
  *(u8 *) (SRAM + offset + i) = '\0';

} // fin saveString()

//////////////////////////////////////////////////////////////////////
// Sauvegarde un entier.
//
//////////////////////////////////////////////////////////////////////
void saveInt(u16 offset, int value)
{
  u8 b1 = (u8) ((value      ) & 0xFF);
  u8 b2 = (u8) ((value >>  8) & 0xFF);
  u8 b3 = (u8) ((value >> 16) & 0xFF);
  u8 b4 = (u8) ((value >> 24)       );

  saveByte(offset + 0, b1);
  saveByte(offset + 1, b2);
  saveByte(offset + 2, b3);
  saveByte(offset + 3, b4);

} // fin saveInt()

//////////////////////////////////////////////////////////////////////
// Charge un octet.
//
//////////////////////////////////////////////////////////////////////
u8 loadByte(u16 offset)
{
  return *(u8 *) (SRAM + offset);

} // fin loadByte()

//////////////////////////////////////////////////////////////////////
// Charge une chane.
//
//////////////////////////////////////////////////////////////////////
void loadString(u16 offset, int length, char *string)
{
  for (int i = 0; i != length; i++)
    string[i] = *(u8 *) (SRAM + offset + i);

} // fin loadString()

//////////////////////////////////////////////////////////////////////
// Charge un entier.
//
//////////////////////////////////////////////////////////////////////
int loadInt(u16 offset)
{
  u8 b1 = loadByte(offset + 0);
  u8 b2 = loadByte(offset + 1);
  u8 b3 = loadByte(offset + 2);
  u8 b4 = loadByte(offset + 3);

  return (int) ((b1 & 0xFF) | ((b2 << 8) & 0xFF00) | ((b3 << 16) & 0xFF0000) | (b4 << 24));

} // fin loadInt()

#endif

// fin saving.h
