#ifndef _GBA_SFX_H
#define _GBA_SFX_H

#include "main.h"

//////////////////////////////////////////////////////////////////////
// TYPES ET DEFINITIONS
//////////////////////////////////////////////////////////////////////

// registre d'effets spciaux sur les couleurs
#define REG_BLDMOD     *(u16*) 0x4000050

// paramtre des effets spciaux
#define REG_COLEV      *(u16*) 0x4000052
#define REG_COLY       *(u16*) 0x4000054

// premire cible
#define TARGET1_BG0    1
#define TARGET1_BG1    2
#define TARGET1_BG2    4
#define TARGET1_BG3    8
#define TARGET1_OBJ    16
#define TARGET1_BD     32

// seconde cible
#define TARGET2_BG0    256
#define TARGET2_BG1    512
#define TARGET2_BG2    1024
#define TARGET2_BG3    2048
#define TARGET2_OBJ    4096
#define TARGET2_BD     8192
#define TARGET2_ALL    TARGET2_BG0 | TARGET2_BG1 | TARGET2_BG2 | TARGET2_BG3 | TARGET2_OBJ | TARGET2_BD

// type d'effet
#define SFX_NONE       0 << 6
#define SFX_ALPHA      1 << 6
#define SFX_FADE_IN    2 << 6
#define SFX_FADE_OUT   3 << 6

#define BLEND(a, b)    (a | (b << 8))

//////////////////////////////////////////////////////////////////////
// PROTOTYPES
//////////////////////////////////////////////////////////////////////

void fadeIn(const u16[], const u16[]);
void fadeOut();

//////////////////////////////////////////////////////////////////////
// IMPLEMENTATIONS
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Provoque un fondu au noir du fond de l'cran en mode 4
//
//////////////////////////////////////////////////////////////////////

void fadeOut()
{
  // fondu au noir sur la palette
  // du fond de l'cran

  // composantes de chaque couleur
  u16 r, g, b;

  // pour les 32 valeurs possibles d'une composante
  for (int i = 0; i < 32; i++)
  {
    // pour chaque couleur de la palette
    for (int j = 0; j < 256; j++)
    {
      // on dcompose la couleur
      r = (palette[j])        & 0x1F;
      g = (palette[j] >> 5)   & 0x1F;
      b = (palette[j] >> 10)  & 0x1F;

      // on se dirige vers le noir
      if (r > 0) r--;
      if (g > 0) g--;
      if (b > 0) b--;

      // on change la palette
      palette[j] = RGB(r, g, b);
    }

    // on lance un timer pour ralentir l'effet
    waitTime(0, 2000 / 32);
  }

} // fin fadeIn()

//////////////////////////////////////////////////////////////////////
// Affiche une image en passant d'un cran noir  l'image.
// L'cran doit tre en mode 4.
//
// param. : picture,          l'image  afficher
// param. : pictapl,          la palette de l'image
//
//////////////////////////////////////////////////////////////////////

void fadeIn(const u16 picture[], const u16 pictpal[])
{
  // on cre une palette de couleurs noire
  for (int i = 0; i < 256; i++)
    palette[i] = 0;

  // on place l'image en mmoire
  REG_DMA3SAD = (u32) picture;
  REG_DMA3DAD = (u32) videoBuffer;
  REG_DMA3CNT = SCREEN_HEIGHT * SCREEN_WIDTH / 2 | DMA_16NOW;

  // composantes de chaque couleur
  u16 r, g, b;

  // pour les 32 valeurs possibles d'une composante
  for (int i = 0; i < 32; i++)
  {
    // pour chaque couleur de la palette
    for (int j = 0; j < 256; j++)
    {
      // on dcompose la couleur
      r = (palette[j]      ) & 0x1F;
      g = (palette[j] >> 5 ) & 0x1F;
      b = (palette[j] >> 10) & 0x1F;

      // on se dirige vers le noir
      if (r < ((pictpal[j]      ) & 0x1F)) r++;
      if (g < ((pictpal[j] >> 5 ) & 0x1F)) g++;
      if (b < ((pictpal[j] >> 10) & 0x1F)) b++;

      // on change la palette
      palette[j] = RGB(r, g, b);
    }

    // on lance un timer pour ralentir l'effet
    waitTime(0, 2000 / 32);
  }

} // fin fadeOut()

#endif

// fin de sfx.h
